# SetMethods - An ad-on R package for advanced QCA
# Ioana-Elena Oana and Carsten Schneider
# Replication Script:

library(SetMethods)

# First rows of the Schneider et al.(2010) data called SCHF in package SetMethods:
data(SCHF)
head(SCHF)

# We create the truth table:

TT_y <- truthTable(SCHF, outcome = "EXPORT",
                   conditions = c("EMP","BARGAIN","UNI",
                                  "OCCUP","STOCK", "MA"),
                   incl.cut = .9,
                   complete = TRUE,
                   PRI = TRUE,
                   sort.by = c("out", "incl", "n"))

# Get the parsimonious solution:

sol_yp <- minimize(TT_y, include = "?", details = TRUE, 
                   show.cases = TRUE)

# Get the intermediate solution:

sol_yi <- minimize(TT_y, include = "?", details = TRUE, 
                   show.cases = TRUE, dir.exp = c(0,0,0,0,0,0))

# Get typical cases for the first term of the second intermediate solution:

mmr (results = sol_yi, outcome = "EXPORT", neg.out = FALSE, 
     sol = "c1p1i2", match = FALSE, cases = 2, term = 1)

# Get deviant cases consistency for the parsimonious solution:
mmr (results = sol_yp, outcome = "EXPORT", neg.out = FALSE, 
     sol = 1, match = FALSE, cases = 3)

# Plot each sufficient path of the parsimonious solution:
pimplot(data = SCHF, results = sol_yp, outcome = "EXPORT", case_labels = FALSE)

# Get deviant cases coverage for the parsimonious solution:

mmr (results = sol_yp, outcome = "EXPORT", neg.out = FALSE, 
     sol = 1, match = FALSE, cases = 4)

# Get individually irrelevant cases for the parsimonious solution:

mmr (results = sol_yp, outcome = "EXPORT", neg.out = FALSE, 
     sol = 1, match = FALSE, cases = 5)

# Get matching pairs of typical and IIR cases for the first term of the parsimonious solution:

mmr (results = sol_yp, outcome = "EXPORT", neg.out = FALSE, 
     sol = 1, match = TRUE, cases = 2, term = 1)

# Get matching pairs of typical and typical cases for the first term of the parsimonious solution:

mmr (results = sol_yp, outcome = "EXPORT", neg.out = FALSE, 
     sol = 1, match = TRUE, cases = 1, term = 1)

# Get matching pairs of typical and deviant consistency cases for the parsimonious solution:

mmr (results = sol_yp, outcome = "EXPORT", neg.out = FALSE, 
     sol = 1, match = TRUE, cases = 3)

# Get matching pairs of deviant coverage and IIR cases for the parsimonious solution:

mmr (results = sol_yp, outcome = "EXPORT", neg.out = FALSE, 
     sol = 1, match = TRUE, cases = 4)

# Ban logical reminders contradicting necessity:
# Let's assume that "STOCK + MA" is necessary for "EXPORT":

newtt <- esa(oldtt = TT_y, nec_cond = "STOCK + MA")

# Only print LRs on screen:

newtt$tt[which(newtt$tt$incl=="-"),]

# Ban impossible logical remainders:
newtt <- esa(oldtt = TT_y, untenable_LR = "BARGAIN*~OCCUP")

# Ban contradictory rows:
newtt <- esa(oldtt = TT_y, contrad_rows = c("19", "14", "46", "51"))

# Assuming the theory can be summarized as "EMP*~MA + STOCK", 
# perform theory evaluation using the second intermediate solution:

theory.evaluation(theory = "EMP*~MA + STOCK", empirics = sol_yi, 
                  outcome = "EXPORT", sol = 2, print.data=FALSE)

# Perfom cluster diagnostic:
# First we need to load the Schneider et. al. (2010) data in the long format:
data(SCHLF)

# This data has a column identifying the unit (country) 
# and the clustering element (year):
head(SCHLF)

# Get the intermediate solution:

sol_yi <- minimize(SCHLF, outcome = "EXPORT",
                   conditions = c("EMP","BARGAIN","UNI",
                                  "OCCUP","STOCK", "MA"),
                   incl.cut1 = .9, 
                   include = "?", 					   
                   details = TRUE, show.cases = TRUE, 
                   dir.exp = c(0,0,0,0,0,0))

# Get pooled, within, and between consistencies for the intermediate solution:

cluster(data = SCHLF, results = sol_yi, outcome = "EXPORT", unit_id = "COUNTRY",
        cluster_id = "YEAR")

# Get pooled, within, and between consistencies for ~EMP as necessary for EXPORT:

cluster(data = SCHLF, results = "~EMP", outcome = "EXPORT", unit_id = "COUNTRY", 
        cluster_id = "YEAR", necessity=TRUE)

# Get pooled, within, and between consistencies for EMP*~MA*STOCK as sufficient for EXPORT:

cluster(data = SCHLF, results = "EMP*~MA*STOCK", outcome = "EXPORT", unit_id = "COUNTRY", 
        cluster_id = "YEAR") 

# Plot the prime implicants of the parsimonious solution:

pimplot(data = SCHF, results = sol_yp, outcome = "EXPORT")

# Plot all truth table rows with a consistency higher than 0.9:

pimplot(data=SCHF, results = sol_yi, incl.tt=0.9, outcome = "EXPORT", sol = 1)

# Plot truth table rows "60" and "61":

pimplot(data=SCHF, results = sol_yi, ttrows =c("60","61"), 
        outcome = "EXPORT", sol = 1)

# For plotting results of necessity analyses using superSubset,
# the first stept is to obtain an "sS" object:

SUPSUB <- superSubset(SCHF, outcome="EXPORT", 
                      conditions = c("EMP","BARGAIN","UNI","OCCUP","STOCK", "MA"), 
                      relation = "necessity", incl.cut = 0.8)
SUPSUB

# This can be imputed as result and necessity should be set to TRUE:

pimplot(data = SCHF, results = SUPSUB, outcome = "EXPORT", necessity = TRUE)

# Display radar chart for the second intermediate solution:
QCAradar(results = sol_yi, outcome = "EXPORT", fit=TRUE, sol = 2)

# Show a radar chart for the following boolean expression "A*~B*C*~D"
QCAradar(results = "A*~B*C*~D")

# Generate fake data
set.seed(4)
x <- runif(20, 0, 1)

# Find quantiles
quant <- quantile(x, c(.2, .4, .5, .6, .8))

# Theoretical calibration
x_cal <- NA  
x_cal[x <= quant[1]] <- 0
x_cal[x > quant[1] & x <= quant[2]] <- .2
x_cal[x > quant[2] & x <= quant[3]] <- .4
x_cal[x > quant[3] & x <= quant[4]] <- .6
x_cal[x > quant[4] & x <= quant[5]] <- .8
x_cal[x > quant[5]] <- 1
x_cal

# Indirect calibration (binomial)
a <- indirectCalibration(x, x_cal, binom = TRUE)
a
